# -----------------------------------------------------------------------------
# Copyright (c) 2019, PyInstaller Development Team.
#
# Distributed under the terms of the GNU General Public License with exception
# for distributing bootloader.
#
# The full license is in the file COPYING.txt, distributed with this software.
# -----------------------------------------------------------------------------

# Hook for MPL-Data-Cast
from PyInstaller.utils.hooks import collect_data_files

hiddenimports = [
    "cytopix",
    "dcnum",
    "dcnum.feat",
    "dcnum.logic",
    "dcnum.meta",
    "dcnum.read",
    "dcnum.segm",
    "dcnum.write",
    "skimage.feature",
]

# Data files
datas = collect_data_files("cytopix", include_py_files=True)
datas += collect_data_files("cytopix", subdir="gui/img")

# Add the Zstandard library used by dclab
datas += collect_data_files("hdf5plugin", includes=["plugins/libh5zstd.*"])



import os


try:
    import pyi_splash
except (ImportError, ModuleNotFoundError):
    pyi_splash = None


def splash_close():
    """Close the splash screen"""
    if pyi_splash is not None:
        if pyi_splash.is_alive():
            pyi_splash.close()

    os.environ["_PYIBoot_SPLASH"] = "handled"



import pathlib
import tomllib

import cytopix
# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

project = 'CytoPix'
github_project = 'DC-analysis/' + project

with (pathlib.Path(__file__).parent.parent / "pyproject.toml").open("rb") as f:
    data = tomllib.load(f)
authors = [a["name"] for a in data["project"]["authors"]]
author = ", ".join(authors)
copyright = '2025, ' + author
release = cytopix.__version__

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = []

templates_path = ['_templates']
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']


# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = 'sphinx_rtd_theme'
html_static_path = ['_static']


# Example configuration for intersphinx: refer to the Python standard library.
intersphinx_mapping = {
    "python": ('https://docs.python.org/', None),
    "dclab": ('https://dclab.readthedocs.io/en/stable/', None),
    "dcnum": ('https://dcnum.readthedocs.io/en/stable/', None),
    "h5py": ('https://h5py.readthedocs.io/en/stable/', None),
    "numpy": ('https://docs.scipy.org/doc/numpy/', None),
    }



import numpy as np
from skimage.color import gray2rgb, hsv2rgb, rgb2hsv


def colorize_image_with_labels(image, labels, saturation=.4, ret_hues=True):
    """Convert a 2D grayscale image to a (label-defined) colored RGB image

    Parameters
    ----------
    image: 2d ndarray unint8
        grayscale image
    labels: 2d ndarray integer
        mask image for colorization
    saturation: float
        color saturation (in HSV)
    ret_hues: bool
        return color hue dictionary
    """
    hsv_img = rgb2hsv(np.array(gray2rgb(image), dtype=np.uint8))
    hue_img = hsv_img[:, :, 0]
    sat_img = hsv_img[:, :, 1]
    value_img = hsv_img[:, :, 2]
    # add the color
    num_labels = np.max(labels)
    hues = np.linspace(0, 1, num_labels, endpoint=False)
    hue_dict = {}
    for ii in range(num_labels):
        mask = labels == ii + 1
        if mask.sum():
            hue_img[mask] = hues[ii]
            sat_img[mask] = saturation
            hue_dict[ii + 1] = hues[ii]
    hsv_img_2 = np.array(np.dstack((hue_img, sat_img, value_img)))
    rgb = hsv2rgb(hsv_img_2)
    rgb = np.array(rgb * 255, dtype=np.uint8)
    if ret_hues:
        return rgb, hue_dict
    else:
        return rgb


def colorize_image_with_mask(image, mask, mask2=None, saturation=.4):
    """Convert a 2D grayscale image to a (masked-defined) colored RGB image

    Parameters
    ----------
    image: 2d ndarray unint8
        grayscale image
    mask: 2d ndarray boolean
        mask image for colorization
    mask2: 2d ndarray boolean
        optional second mask image for comparison with first mask image
    saturation: float
        color saturation (in HSV)
    """
    hsv_img = rgb2hsv(np.array(gray2rgb(image), dtype=np.uint8))
    hue_img = hsv_img[:, :, 0]
    sat_img = hsv_img[:, :, 1]
    value_img = hsv_img[:, :, 2]
    hue_img[mask] = .5
    hue_img[~mask] = .2
    if mask2 is not None:
        mask_diff = np.array(mask, dtype=int) - mask2
        hue_img[mask_diff < 0] = 0
        hue_img[mask_diff > 0] = .8
    sat_img[:] = saturation
    hsv_img_2 = np.array(np.dstack((hue_img, sat_img, value_img)))
    rgb = hsv2rgb(hsv_img_2)
    rgb = np.array(rgb * 256, dtype=np.uint8)
    return rgb



import errno
import hashlib
import logging
import os
import pathlib
import itertools

import dclab
from dcnum import write
import h5py
import numpy as np
from PIL import Image

from .seg_session import SegmentationSession

logger = logging.getLogger(__name__)


def dc_to_png_files(dc_path: pathlib.Path | str,
                    png_dir: pathlib.Path | str,
                    export_labels: bool = False,
                    indices: np.ndarray = None,
                    ):
    """Export images from an .rtdc file to PNG

    Parameters
    ----------
    dc_path: pathlib.Path
        Input .rtdc file
    png_dir: pathlib.Path
        PNG files output directory
    export_labels: bool
        If specified, save uint8 label images alongside input images
        (with additional `_label` stem suffix) using the .dcseg session file.
    indices: np.ndarray
        Only export images with given indices
    """
    dc_path = pathlib.Path(dc_path)
    png_dir = pathlib.Path(png_dir)
    png_dir.mkdir(exist_ok=True, parents=True)

    if export_labels:
        ses = SegmentationSession(dc_path)
    else:
        ses = None

    with dclab.new_dataset(dc_path) as ds:

        if indices is None:
            indices = np.arange(len(ds))
        digits = len(str(int(np.max(indices))))

        for idx in indices:
            im = Image.fromarray(ds["image"][idx])
            im.save(png_dir / f"image_{str(idx).zfill(digits)}.png")
            if ses is not None:
                # store label image
                frame = ds["frame"][idx]
                larr = ses.get_labels(frame)
                # normalize
                # limit in case there are >255 labels
                max_val = min(larr.max(), 255)
                larr = np.array(larr / max_val * 255, dtype=np.uint8)
                lim = Image.fromarray(larr)
                lim.save(png_dir / f"image_{str(idx).zfill(digits)}_label.png")


def png_files_to_dc(png_paths: list | str | pathlib.Path,
                    dc_path: pathlib.Path):
    """Convert a set of PNG files to an .rtdc file

    Parameters
    ----------
    png_paths:
        List of PNG files or a directory containing PNG files.
        If a directory is specified, then it is search for PNG files
        recursively.
    dc_path:
        Path to an .rtdc file that is created from the .png files.
        If the .rtdc file already exists and the input file names and
        sizes match that of the path specified, then the existing file
        is used. Otherwise, a FileExistsError is raised.
    """
    # Get list of input PNG files
    input_files = []
    if isinstance(png_paths, (pathlib.Path, str)):
        path = pathlib.Path(png_paths)
        if path.is_dir():
            input_files += path.rglob("*.png", case_sensitive=False)
        else:
            input_files += [png_paths]
    else:
        input_files += list(png_paths)
    input_files = sorted(input_files)

    if not input_files:
        raise ValueError("No PNG files specified or found!")

    logger.info(f"Loading {len(input_files)} PNG files...")

    # Get a list of file names and file sizes
    png_sizes = [pp.stat().st_size for pp in input_files]
    png_names = [pp.name for pp in input_files]

    # Compute a unique hash of the input files
    hasher = hashlib.md5()
    hasher.update(
        "".join([f"{info}" for info in zip(png_names, png_sizes)]).encode())
    png_hash = hasher.hexdigest()

    # If the output file already exists, check whether the hashes match.
    log_name = "cytopix-png-files"
    if dc_path.exists():
        file_matches = False
        with h5py.File(dc_path) as h5:
            if (log_name in h5["logs"]
                    and h5["logs"][log_name].attrs["hash"] == png_hash):
                # We can reuse this file
                logger.info(f"Reusing existing file {dc_path}")
                file_matches = True

        if not file_matches:
            logger.info(f"Cannot use existing {dc_path} (hash mismatch)")
            raise FileExistsError(
                errno.EEXIST, os.strerror(errno.EEXIST), str(dc_path))
    else:
        # This is the actual workload of this function. Populate the .rtdc
        # file with the image data.
        logger.info(f"Writing .rtdc file {dc_path}")
        with write.HDF5Writer(dc_path) as hw:
            # store the input file information as a log
            log_ds = hw.store_log(
                log=log_name,
                data=[f"{d[0]} {d[1]}" for d in zip(input_files, png_sizes)]
            )
            log_ds.attrs["hash"] = png_hash

            num_images = len(input_files)
            hw.store_feature_chunk(
                feat="frame",
                data=np.arange(1, num_images+1),
            )

            # store the image data to the output file
            image_size = Image.open(input_files[0]).size
            chunk_size = hw.get_best_nd_chunks(item_shape=image_size,
                                               feat_dtype=np.uint8)[0]

            for indices in itertools.batched(range(num_images), chunk_size):
                image_data = []
                for ii in indices:
                    im = np.array(Image.open(input_files[ii]), dtype=np.uint8)
                    if len(im.shape) == 3:
                        # convert RGB to grayscale by taking the red channel
                        im = im[:, :, 0]
                    image_data.append(im)
                # store the image chunk
                hw.store_feature_chunk(
                    feat="image",
                    data=np.array(image_data, dtype=np.uint8),
                )



import pathlib

import numpy as np

from cytopix import seg_session


data_path = pathlib.Path(__file__).parent / 'data'


def test_basic_session(tmp_path):
    """Test basic session functionality"""
    # open session
    ses = seg_session.SegmentationSession(
        path_dc=data_path / "blood_minimal.rtdc",
        path_session=tmp_path / "blood_minimal_session.dcseg",
    )
    assert not ses.complete

    # write a label image to the file
    labels = ses.get_labels(240105)
    labels[:] = 0
    labels[:, 0] = 1
    labels[:, 2] = 2
    labels[:, 3] = 3

    ses.write_user_labels(frame=240105, labels=labels)

    assert np.all(ses.get_labels(240105) == labels)



import logging


class DevNull:
    """Effectively a file-like object for piping everything to nothing."""

    def write(self, *args, **kwargs):
        pass


class PrettyFormatter(logging.Formatter):
    grey = "\x1b[38;20m"
    yellow = "\x1b[33;20m"
    red = "\x1b[31;20m"
    bold_red = "\x1b[31;1m"
    reset = "\x1b[0m"
    format = "%(levelname)s %(name)s: %(message)s"

    FORMATS = {
        logging.DEBUG: grey + format + reset,
        logging.INFO: grey + format + reset,
        logging.WARNING: yellow + format + reset,
        logging.ERROR: red + format + reset,
        logging.CRITICAL: bold_red + format + reset
    }

    def format(self, record):
        log_fmt = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_fmt)
        return formatter.format(record)


def main():
    from importlib import resources
    import multiprocessing as mp
    import sys
    from PyQt6 import QtWidgets, QtCore, QtGui

    mp.freeze_support()

    # In case we have a frozen application, and we encounter errors
    # in subprocesses, then these will try to print everything to stdout
    # and stderr. However, if we compiled the app with PyInstaller with
    # the --noconsole option, sys.stderr and sys.stdout are None and
    # an exception is raised, breaking the program.
    if sys.stdout is None:
        sys.stdout = DevNull()
    if sys.stderr is None:
        sys.stderr = DevNull()

    # Tell the root logger to pretty-print logs
    root_logger = logging.getLogger()
    root_logger.setLevel(logging.INFO)
    handler = logging.StreamHandler()
    handler.setLevel(logging.INFO)
    handler.setFormatter(PrettyFormatter())
    root_logger.addHandler(handler)

    from .main import CytoPix

    app = QtWidgets.QApplication(sys.argv)
    ref_ico = resources.files("cytopix.img") / "cytopix_icon.png"
    with resources.as_file(ref_ico) as path_icon:
        app.setWindowIcon(QtGui.QIcon(str(path_icon)))

    # Use dots as decimal separators
    QtCore.QLocale.setDefault(QtCore.QLocale(QtCore.QLocale.c()))

    window = CytoPix(*app.arguments()[1:])  # noqa: F841

    sys.exit(app.exec())



import colorsys
from importlib import import_module
import logging
import pathlib
import signal
import sys
import traceback
import webbrowser

from dcnum.meta import paths as dcnum_paths
from dcnum.segm import get_available_segmenters
import numpy as np
from PyQt6 import QtCore, QtWidgets
from PyQt6.QtGui import QKeySequence, QShortcut
import pyqtgraph as pg
from scipy.ndimage import binary_fill_holes
from skimage import segmentation


from ._version import version
from .main_ui import Ui_MainWindow
from . import colorize
from . import png_io
from . import seg_session
from . import splash


pg.setConfigOptions(imageAxisOrder='row-major')


class CytoPix(QtWidgets.QMainWindow):

    def __init__(self, *arguments):
        """Initialize CytoPix GUI

        If you pass the "--version" command line argument, the
        application will print the version after initialization
        and exit.
        """
        super(QtWidgets.QMainWindow, self).__init__()
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

        # Disable native menu bar (e.g. on macOS)
        self.ui.menubar.setNativeMenuBar(False)

        self.wid = self.ui.graphics_widget

        self.v1a = self.wid.addViewBox(row=1, col=0, lockAspect=True)
        self.v1a.setDefaultPadding(0)

        self.pg_image = pg.ImageItem(np.arange(80 * 320).reshape((80, -1)))

        self.v1a.addItem(self.pg_image)
        self.v1a.disableAutoRange('xy')
        self.v1a.autoRange()

        kern = np.array([[0]])
        self.pg_image.setDrawKernel(kern, mask=kern, center=(0, 0),
                                    mode=self.on_draw)

        self.pg_image.setLevels([1, 80*320])

        #: current visualization state:
        #: - 0: show labels with different colors
        #: - 1: show all labels as one color
        self.vis_state = 0

        # Current image as grayscale
        self.image = None
        self.image_bg = None
        self.subtract_bg = True
        self.labels = None
        self.show_labels = True
        self.current_drawing_label = 1  # 1 to 9
        self.auto_contrast = True
        self.label_saturation = 0.4
        self.segses = None

        # Settings are stored in the .ini file format. Even though
        # `self.settings` may return integer/bool in the same session,
        # in the next session, it will reliably return strings. Lists
        # of strings (comma-separated) work nicely though.
        QtCore.QCoreApplication.setOrganizationName("DC-Analysis")
        QtCore.QCoreApplication.setOrganizationDomain("mpl.mpg.de")
        QtCore.QCoreApplication.setApplicationName("CytoPix")
        QtCore.QSettings.setDefaultFormat(QtCore.QSettings.Format.IniFormat)
        #: CytoPix settings
        self.settings = QtCore.QSettings()

        # register search paths with dcnum
        for path in self.settings.value("segm/torch_model_files", []):
            path = pathlib.Path(path)
            if path.is_dir():
                dcnum_paths.register_search_path("torch_model_files", path)

        self.logger = logging.getLogger(__name__)

        # GUI
        self.setWindowTitle(f"CytoPix {version}")

        # File menu
        self.ui.actionSegmentRrtdcFile.triggered.connect(
            self.on_action_segment_rtdc)
        self.ui.actionSegmentPngImages.triggered.connect(
            self.on_action_segment_png)
        self.ui.actionExportImages.triggered.connect(
            self.on_action_export_images)
        self.ui.actionQuit.triggered.connect(self.on_action_quit)
        # Help menu
        self.ui.actionDocumentation.triggered.connect(self.on_action_docs)
        self.ui.actionSoftware.triggered.connect(self.on_action_software)
        self.ui.actionAbout.triggered.connect(self.on_action_about)

        cwid = self.centralWidget()

        # Shortcuts
        # Keyboard shortcuts
        # navigate next
        self.shortcut_next = QShortcut(QKeySequence('Right'), cwid)
        self.shortcut_next.activated.connect(self.goto_next)
        # navigate previous
        self.shortcut_prev = QShortcut(QKeySequence('Left'), cwid)
        self.shortcut_prev.activated.connect(self.goto_prev)
        # hide mask
        self.shortcut_toggle_cont = QShortcut(QKeySequence('Space'), cwid)
        self.shortcut_toggle_cont.activated.connect(self.toggle_mask)
        # visualization state
        self.shortcut_toggle_vis = QShortcut(QKeySequence('V'), cwid)
        self.shortcut_toggle_vis.activated.connect(self.next_visualization)
        # auto-contrast
        self.shortcut_toggle_contr = QShortcut(QKeySequence('C'), cwid)
        self.shortcut_toggle_contr.activated.connect(self.toggle_contrast)
        # background correction
        self.shortcut_toggle_bg = QShortcut(QKeySequence('B'), cwid)
        self.shortcut_toggle_bg.activated.connect(self.toggle_background)
        # Current label
        self.shortcuts_label = []
        for ii in range(1, 10):
            sci = QShortcut(QKeySequence(str(ii)), cwid)
            sci.activated.connect(self.change_drawing_label)
            self.shortcuts_label.append(sci)
        # Label saturation
        self.shortcut_plus = QShortcut(QKeySequence('.'), cwid)
        self.shortcut_plus.activated.connect(self.saturation_plus)
        self.shortcut_minus = QShortcut(QKeySequence('-'), cwid)
        self.shortcut_minus.activated.connect(self.saturation_minus)

        # if "--version" was specified, print the version and exit
        if "--version" in arguments:
            print(version)
            QtWidgets.QApplication.processEvents(
                QtCore.QEventLoop.ProcessEventsFlag.AllEvents, 300)
            sys.exit(0)

        splash.splash_close()

        # finalize
        self.show()
        self.activateWindow()
        self.setWindowState(QtCore.Qt.WindowState.WindowActive)
        self.showMaximized()

        for arg in arguments:
            if isinstance(arg, str):
                pp = pathlib.Path(arg)
                if pp.suffix in [".rtdc", ".dc"]:
                    self.on_action_segment_rtdc(pp)
                    break
                elif pp.is_dir():
                    self.on_action_segment_png(pp)
                    break

    @QtCore.pyqtSlot(QtCore.QEvent)
    def dragEnterEvent(self, e):
        """Whether files are accepted"""
        if e.mimeData().hasUrls():
            e.accept()
        else:
            e.ignore()

    @QtCore.pyqtSlot(QtCore.QEvent)
    def dropEvent(self, e):
        """Add dropped files to view"""
        urls = sorted(e.mimeData().urls())
        pathlist = [pathlib.Path(ff.toLocalFile()) for ff in urls]
        if pathlist:
            # check whether the first file is a DC file
            pp0 = pathlist[0]
            if pp0.is_file() and pp0.suffix in [".rtdc", ".dc"]:
                self.on_action_segment_rtdc(pp0)
                return
            # we have a list of PNG files or a directory
            png_files = [pp for pp in pathlist
                         if pp.is_file() and pp.suffix == ".png"]
            if png_files:
                self.on_action_segment_png(png_files)
            else:
                # recurse into the directory
                png_files_recursive = []
                for pp in pathlist:
                    if pp.is_dir():
                        png_files_recursive += sorted(pp.rglob("*.png"))
                self.on_action_segment_png(png_files_recursive)

    @QtCore.pyqtSlot()
    def on_action_about(self) -> None:
        """Show imprint."""
        gh = "DC-analysis/CytoPix"
        rtd = "cytopix.readthedocs.io"
        about_text = (
            f"CytoPix. GUI for pixel-based manual segmentation of DC images."
            f"<br><br>"
            f"Author: Paul Müller and others<br>"
            f"GitHub: "
            f"<a href='https://github.com/{gh}'>{gh}</a><br>"
            f"Documentation: "
            f"<a href='https://{rtd}'>{rtd}</a><br>")  # noqa 501
        QtWidgets.QMessageBox.about(self,
                                    f"CytoPix {version}",
                                    about_text)

    @QtCore.pyqtSlot()
    def on_action_export_images(self, path=None):
        if path is None:
            path = QtWidgets.QFileDialog.getExistingDirectory(
                self,
                "Select directory for PNG export",
            )
        if path:
            png_io.dc_to_png_files(
                dc_path=self.segses.path_dc,
                png_dir=path,
                export_labels=True,
                indices=self.segses.get_labeled_indices(),
            )

    @QtCore.pyqtSlot()
    def on_action_segment_png(self, path=None):
        """Open a dialog to load a directory of PNG files"""
        if path is None:
            path = QtWidgets.QFileDialog.getExistingDirectory(
                self,
                'Select directory containing PNG images',
                )
        if path:
            path = pathlib.Path(path)
            # convert directory of PNG images to .rtdc
            dc_path = path.with_name(path.name + ".rtdc")
            png_files = [p for p in path.glob("*.png")
                         if not p.name.endswith("_label.png")]
            try:
                png_io.png_files_to_dc(png_files, dc_path)
            except FileExistsError:
                QtWidgets.QMessageBox.critical(
                    self,
                    "PNG session inconsistency",
                    f"CytoPix detected an inconsistency while comparing the "
                    f"PNG files in '{path}' and the .rtdc file generated "
                    f"in a previous session.\n\n"
                    f"This usually means that somebody modified the PNG files "
                    f"or added/removed new files. This is a problem, because "
                    f"CytoPix creates a static .rtdc file from these PNG "
                    f"files.\n\n "
                    f"To resolve this issue, you can either "
                    f"\n- delete the file '{dc_path}' and its associated "
                    f"session file '{dc_path.with_suffix('.dcseg').name}' "
                    f"(This will destroy your previous session),"
                    f"\n- restore the PNG directory to its previous state, or"
                    f"\n- directly open '{dc_path}' instead."
                )
            else:
                # open session
                self.open_session(dc_path)

    @QtCore.pyqtSlot()
    def on_action_segment_rtdc(self, path=None):
        """Open dialog to add a single .rtdc file"""
        if path is None:
            path, _ = QtWidgets.QFileDialog.getOpenFileName(
                self,
                'Select DC data',
                '',
                'RT-DC data (*.rtdc)')
        if path:
            # open session
            self.open_session(path)

    @QtCore.pyqtSlot()
    def on_action_docs(self):
        webbrowser.open("https://cytopix.readthedocs.io")

    @QtCore.pyqtSlot()
    def on_action_software(self) -> None:
        """Show used software packages and dependencies."""
        libs = ["dcnum",
                "h5py",
                "numpy",
                "pillow",
                "pyqtgraph",
                "torch",
                ]

        sw_text = f"CytoPix {version}\n\n"
        sw_text += f"Python {sys.version}\n\n"
        sw_text += "Modules:\n"
        for lib in libs:
            try:
                mod = import_module(lib)
            except ImportError:
                pass
            else:
                sw_text += f"- {mod.__name__} {mod.__version__}\n"
        sw_text += f"- PyQt6 {QtCore.QT_VERSION_STR}\n"

        QtWidgets.QMessageBox.information(self, "Software", sw_text)

    @QtCore.pyqtSlot()
    def on_action_quit(self) -> None:
        """Determine what happens when the user wants to quit"""
        self.save_labels()
        QtCore.QCoreApplication.quit()

    def open_session(self, path):
        segmenter_class = get_available_segmenters()["thresh"]
        segmenter_kwargs = {
            "kwargs_mask": {
                "clear_border": False,  # for training, we need all events
                "fill_holes": True,
                "closing_disk": 1
            }
        }
        self.segses = seg_session.SegmentationSession(
            path_dc=path,
            segmenter_class=segmenter_class,
            segmenter_kwargs=segmenter_kwargs)
        self.show_event(*self.segses.get_next_frame())
        # give graphics widget mouse/event focus
        self.wid.setFocus()

    @QtCore.pyqtSlot()
    def closeEvent(self, event):
        """Determine what happens when the user wants to quit"""
        self.save_labels()
        event.accept()

    @QtCore.pyqtSlot()
    def change_drawing_label(self):
        sender = self.sender()
        self.current_drawing_label = int(sender.key().toString())
        self.ui.label_label_current.setText(
            f"<b>{self.current_drawing_label}</b>")

    def get_labels_from_ui(self):
        return np.copy(self.labels) if self.labels is not None else None

    @QtCore.pyqtSlot()
    def goto_next(self):
        """Go to next unlabeled event"""
        self.save_labels()
        if self.segses:
            # get the next event
            self.show_event(*self.segses.get_next_frame())

    @QtCore.pyqtSlot()
    def goto_prev(self):
        """Go one frame back"""
        self.save_labels()
        if self.segses:
            self.show_event(*self.segses.get_prev_frame())

    def on_draw(self, dk, image, mask, ss, ts, ev):
        """Called when the user draws"""
        if self.image is None:
            self.logger.warning("No image data, not drawing")
            return
        elif self.show_labels is None:
            return

        if hasattr(ev, "isFinish"):
            # drag ended
            finish = ev.isFinish()
        else:
            # only single click
            finish = True

        fill_holes = finish
        # Set the pixel value accordingly.
        mdf = ev.modifiers().value
        if mdf == (QtCore.Qt.Modifier.SHIFT | QtCore.Qt.Modifier.ALT).value:
            # delete the mask at that location
            segmentation.flood_fill(self.labels,
                                    seed_point=(ts[0].start, ts[1].start),
                                    new_value=0,
                                    in_place=True)
        else:
            delete = mdf == QtCore.Qt.Modifier.SHIFT.value
            value = 0 if delete else self.current_drawing_label
            self.labels[ts] = value

        if finish:
            # redraw all labels
            self.update_plot(fill_holes=fill_holes)
        else:
            # only draw white points where user dragged mouse
            image_rgb = np.array(self.pg_image.image, dtype=int)
            image_rgb[ts] = 250
            image_rgb = np.array(np.clip(image_rgb, 0, 255), dtype=np.uint8)
            self.update_plot(image_rgb=image_rgb, draw_labels=False)

    def saturation_minus(self):
        self.label_saturation -= .05
        self.label_saturation = max(self.label_saturation, .05)
        self.update_plot()  # inefficient, but no optimization necessary

    def saturation_plus(self):
        self.label_saturation += .05
        self.label_saturation = min(self.label_saturation, 1)
        self.update_plot()  # inefficient, but no optimization necessary

    def save_labels(self):
        """Save the current contour in the session"""
        if self.segses:
            frame = self.segses.current_frame
            labels = self.get_labels_from_ui()
            if labels is not None:
                self.segses.write_user_labels(frame, labels)

    def show_event(self, image, image_bg, labels, frame):
        self.image = np.array(image, dtype=int)
        self.image_bg = image_bg
        self.labels = np.array(labels, dtype=np.uint8)

        # reset the view
        self.v1a.setRange(
            xRange=(0, self.image.shape[1]),
            yRange=(0, self.image.shape[0]))

        self.update_plot()

        if self.segses:
            totframes = len(self.segses.unique_frames)
            self.ui.lineEdit_source.setText(str(self.segses.path_dc))
            self.ui.lineEdit_session.setText(str(self.segses.path_session))
            self.ui.label_frame.setText(
                f"{self.segses.current_index_unique + 1}/{totframes}"
            )

    @QtCore.pyqtSlot()
    def toggle_background(self):
        self.subtract_bg = not self.subtract_bg
        self.update_plot()

    @QtCore.pyqtSlot()
    def toggle_contrast(self):
        self.auto_contrast = not self.auto_contrast
        self.update_plot()

    @QtCore.pyqtSlot()
    def update_plot(self, fill_holes=False, draw_labels=True, image_rgb=None):
        """Update plot in case visualization changed

        The number of calls to this function should be minimized.
        However, since we are dealing with small images, anything
        to reduce the number of calls is premature optimization.
        """
        if self.image is None:
            self.logger.warning("Not updating plot, no data loaded")
            return
        if self.subtract_bg:
            image = self.image - self.image_bg + 128
        else:
            image = self.image

        if image_rgb is not None:
            pass
        elif self.show_labels and draw_labels:
            if self.vis_state == 0:
                self.ui.widget_labels.setVisible(True)
                labels = self.labels
                if fill_holes:
                    for ii in range(1, labels.max() + 1):
                        maski = labels == ii
                        labels[binary_fill_holes(maski)] = ii
            else:
                self.ui.widget_labels.setVisible(False)
                labels = self.labels > 0
                if fill_holes:
                    labels = binary_fill_holes(labels)
            # draw RGB image
            image_rgb, hues = colorize.colorize_image_with_labels(
                image,
                labels=labels,
                saturation=self.label_saturation,
                ret_hues=True)
            colab = "Frame Labels: "
            for ii, lid in enumerate(hues):
                r, g, b = np.array(colorsys.hsv_to_rgb(hues[lid], 1, 1)) * 255
                color = f"#{int(r):02x}{int(g):02x}{int(b):02x}"
                cur_lab = (f"<span "
                           f"style='color:{color}; background-color:black'"
                           f"><b>{lid}</b></span>")
                colab += cur_lab

                if ii + 1 == self.current_drawing_label:
                    self.ui.label_label_current.setText(cur_lab)

            self.ui.label_labels_available.setText(colab)

        else:
            # draw grayscale image
            image_rgb = image

        (rx1, rx2), (ry1, ry2) = np.array(self.v1a.viewRange(), dtype=int)
        cropped = image[slice(max(0, ry1), ry2), slice(max(0, rx1), rx2)]

        if self.auto_contrast and cropped.size:
            levels = (min(120, cropped.min()), max(136, cropped.max()))
        else:
            levels = (0, 255)

        # adjust contrast according to currently visible area
        kwargs = dict(
            levels=levels,
            levelMode="mono"
        )

        self.pg_image.setImage(image_rgb, **kwargs)

    @QtCore.pyqtSlot()
    def next_visualization(self):
        self.vis_state = (self.vis_state + 1) % 2
        self.update_plot()

    @QtCore.pyqtSlot()
    def toggle_mask(self):
        self.show_labels = not self.show_labels
        self.update_plot()


def excepthook(etype, value, trace):
    """
    Handler for all unhandled exceptions.

    :param `etype`: the exception type (`SyntaxError`,
        `ZeroDivisionError`, etc...);
    :type `etype`: `Exception`
    :param string `value`: the exception error message;
    :param string `trace`: the traceback header, if any (otherwise, it
        prints the standard Python header: ``Traceback (most recent
        call last)``.
    """
    vinfo = f"Unhandled exception in CytoPix version {version}:\n"
    tmp = traceback.format_exception(etype, value, trace)
    exception = "".join([vinfo]+tmp)
    try:
        # Write to the control logger, so errors show up in the
        # cytopix-warnings log.
        main = get_main()
        main.control.logger.error(exception)
    except BaseException:
        # If we send things to the logger and everything is really bad
        # (e.g. cannot write to output hdf5 file or so, then we silently
        # ignore this issue and only print the error message below.
        pass
    QtWidgets.QMessageBox.critical(
        None,
        "CytoPix encountered an error",
        exception
    )


def get_main():
    app = QtWidgets.QApplication.instance()
    for widget in app.topLevelWidgets():
        if isinstance(widget, QtWidgets.QMainWindow):
            return widget


# Make Ctr+C close the app
signal.signal(signal.SIGINT, signal.SIG_DFL)
# Display exception hook in separate dialog instead of crashing
sys.excepthook = excepthook



# Create inno setup .iss file
import io
import os.path as op
import platform
import sys

import cytopix

# make library available
cdir = op.dirname(op.abspath(__file__))
pdir = op.dirname(cdir)
sys.path.insert(0, pdir)

# get version
version = cytopix.__version__

# read dummy
with io.open(op.join(cdir, "win_cytopix.iss_dummy"), 'r') as dummy:
    iss = dummy.readlines()

# replace keywords
for i in range(len(iss)):
    if iss[i].strip().startswith("#define MyAppVersion"):
        iss[i] = '#define MyAppVersion "{:s}"\n'.format(version)
    if iss[i].strip().startswith("#define MyAppPlatform"):
        # sys.maxint returns the same for windows 64bit verions
        iss[i] = '#define MyAppPlatform "win_{}"\n'.format(
            platform.architecture()[0])

# write iss
with io.open(op.join(cdir, "win_cytopix.iss"), 'w') as issfile:
    issfile.writelines(iss)



import logging
import pathlib
from typing import Dict

import dclab
from dcnum.segm import MPOSegmenter, Segmenter, get_available_segmenters
import h5py
import numpy as np

from skimage import measure


logger = logging.getLogger(__name__)


class SegmentDisabled(MPOSegmenter):
    requires_background_correction = False
    mask_postprocessing = True
    mask_default_kwargs = {
        "clear_border": False,
        "fill_holes": False,
        "closing_disk": 0,
    }

    def segment_algorithm(self, image):
        return np.zeros_like(image, dtype=bool)


get_available_segmenters.cache_clear()
_def_methods = get_available_segmenters()


class SegmentationSession:
    def __init__(self,
                 path_dc: pathlib.Path | str,
                 path_session: pathlib.Path | str = None,
                 segmenter_class: Segmenter = None,
                 segmenter_kwargs: Dict = None,
                 ):
        """Create and manipulate .dcseg manual segmentation session files

        In contrast to regular .rtdc files, mask data in .dcseg
        files are stored as np.uint8 arrays where each integer
        value corresponds to one mask. Since the user can
        freely choose with which label to draw, there may be multiple
        masks with the same label identifier. This class helps
        to manage these cases.

        Notes
        -----
        If the input .rtdc file used for labeling contains duplicate
        frames, then only one label image is used in the .dcseg file,
        the other image will be all-zero.

        Everything should still work if the frames in the input .rtdc
        file are not in order, but this is untested.
        """
        path_dc = pathlib.Path(path_dc)
        if path_session is None:
            path_session = path_dc.with_suffix(".dcseg")
        if path_session.suffix != ".dcseg":
            path_session = path_session.with_name(path_session.name + ".dcseg")

        if segmenter_class is None:
            segmenter_class = _def_methods["thresh"]

        # read-in session information
        self.ds = dclab.new_dataset(path_dc)
        self.path_session = path_session
        with h5py.File(path_session, "a") as h5:
            size = len(self.ds)
            sx, sy = self.ds["image"][0].shape
            # label information
            h5.require_dataset(name="user_labels",
                               shape=(size, sx, sy),
                               # not more than 254 events in a frame
                               dtype=np.uint8)
            # indicates how many labels are processed (int) or invalid (np.nan)
            h5.require_dataset(name="events_processed",
                               shape=(len(self.ds),),
                               dtype=float)

        self.path_dc = path_dc
        self.path_session = path_session
        # whether segmentation is complete
        self._complete = False
        #: list of unique frames in the original .rtdc file
        self.unique_frames = sorted(
            np.unique(np.array(self.ds["frame"][:], dtype=np.uint64)))
        #: current frame of interest
        self.current_frame = None
        #: current event of interest in the current frame
        self.current_event_in_frame = 0

        # figure out how we want to do mask processing
        self.segm_kwargs = segmenter_kwargs or {}

        self.segm_class = segmenter_class
        self.segm = self.segm_class(**self.segm_kwargs)
        self.segm_func = self.segm.segment_algorithm_wrapper()

        logger.info(f"Segmenter class is: {self.segm_class}")
        logger.info(f"Segmenter kwargs: {self.segm_kwargs}")

    def __len__(self):
        """Number of events in the original .rtdc dataset"""
        return len(self.ds)

    @property
    def current_index(self):
        """Current index in the original dataset

        If there are duplicate frames in the original dataset,
        then this always returns the first index.
        """
        if self.current_frame is None:
            self.current_frame = self.unique_frames[0]
        return self.get_index(self.current_frame)

    @property
    def current_index_unique(self):
        """Current index in `self.unique_frames`

        Here we are not enumerating the original dataset, but
        the unique frames therein.
        """
        if self.current_frame is None:
            self.current_frame = self.unique_frames[0]
        return self.unique_frames.index(self.current_frame)

    @property
    def complete(self):
        """Property indicating whether the current session is complete"""
        if not self._complete:
            with h5py.File(self.path_session, "a") as h5:
                if np.nanmin(h5["events_processed"][:] > 0) > 0:
                    self._complete = True
        return self._complete

    def get_image_bg(self, frame: int):
        """Return background image"""
        index = self.get_index(frame)
        if "image_bg" in self.ds:
            image_bg = self.ds["image_bg"][index]
        else:
            image_bg = np.full(self.ds["image"].shape[1:],
                               np.median(self.ds["image"][index]))
        return image_bg

    def get_event(self,
                  frame: int,
                  event_in_frame: int = 0):
        """Get event data for this index"""
        frame = int(frame)
        index = self.get_index(frame)
        image = self.ds["image"][index]
        image_bg = self.get_image_bg(frame)
        # get the already-processed contour, otherwise the original one
        mask = self.get_mask(frame, event_in_frame)
        self.current_frame = frame
        self.current_event_in_frame = event_in_frame
        return image, image_bg, mask, frame

    def get_index(self, frame: int):
        """Return the index in the dataset given a frame number"""
        if frame is None:
            index = -1
        else:
            cands = np.where(self.ds["frame"] == frame)[0]
            if cands.size == 0:
                raise IndexError(f"Could not find frame {frame}!")
            index = cands[0]
        return index

    def get_mask(self, frame: int, event_in_frame: int):
        """Return mask of an event (event index starts at 0) in a frame"""
        index = self.get_index(frame)
        with h5py.File(self.path_session) as h5:
            mask = h5["user_labels"][index] == event_in_frame + 1
        return mask

    def get_labels(self, frame: int):
        """Return full integer label image for a frame index"""
        index = self.get_index(frame)
        with h5py.File(self.path_session) as h5:
            labels = h5["user_labels"][index][:]
            if np.sum(labels) == 0:  # user did not edit this frame
                # retrieve image with optional background correction
                if self.segm_class.requires_background_correction:
                    image = (np.array(self.ds["image"][index], dtype=int)
                             - self.get_image_bg(frame))
                else:
                    image = self.ds["image"][index]
                mask = self.segm_func(image)
                # perform optional mask postprocessing
                if self.segm_class.mask_postprocessing:
                    mask = self.segm_class.process_mask(
                        mask, **self.segm.kwargs_mask)
                # label the masks
                labels = measure.label(mask, background=0)
        return labels

    def get_labeled_indices(self):
        """Return indices in `path_dc` that have been labeled"""
        with h5py.File(self.path_session, "a") as h5:
            processed = h5["events_processed"][:]

        # We have to remove all indices that have not been procecces
        indices = np.arange(len(processed))
        used = np.ones_like(indices, dtype=bool)
        used[np.isnan(processed)] = False
        return indices[used]

    def get_next_frame(self):
        """Get the next unedited events (fallback to +1 if all were edited)"""
        if self.current_frame is None:
            frame = self.unique_frames[0]
        else:
            # If there are two consecutive frames that are identical, we
            # have to skip to the second-next index.
            curidx = self.unique_frames.index(self.current_frame)
            nextidx = (curidx + 1) % len(self.unique_frames)
            frame = self.unique_frames[nextidx]

        image, image_bg, _, _ = self.get_event(frame)
        labels = self.get_labels(frame)
        return image, image_bg, labels, frame

    def get_prev_frame(self):
        """Get the events from the previous frame"""
        curidx = self.unique_frames.index(self.current_frame)
        nextidx = (curidx - 1) % len(self.unique_frames)
        frame = self.unique_frames[nextidx]

        image, image_bg, _, _ = self.get_event(frame)
        labels = self.get_labels(frame)
        return image, image_bg, labels, frame

    def invalidate_frame(self, frame: int):
        """That particular image cannot be processed"""
        index = self.get_index(frame)
        with h5py.File(self.path_session, "a") as h5:
            h5["events_processed"][index] = np.nan

    def write_user_labels(self,
                          frame: int,
                          labels: np.ndarray):
        """Write the user-defined labels to the session file"""
        frame = int(frame)
        index = self.get_index(frame)
        with h5py.File(self.path_session, "a") as h5:
            h5["user_labels"][index] = labels
            h5["events_processed"][index] = np.max(labels)

    def write_user_mask(self,
                        frame: int,
                        event_in_frame: int,
                        mask: np.ndarray):
        """Write the mask data of one event in a frame to the session file

        This overrides any previously labeling data for this particular
        event but preserves other labeling information (unless there is
        overlap).
        """
        frame = int(frame)
        index = self.get_index(frame)
        with h5py.File(self.path_session, "a") as h5:
            # Only save one single mask
            h5["events_processed"][index] = max(
                h5["events_processed"][index], event_in_frame + 1)
            label = event_in_frame + 1
            label_img = h5["user_labels"][index]
            label_img[label_img == label] = 0
            label_img[mask] = label
            h5["user_labels"][index] = label_img



import pathlib
import shutil

import dclab
import numpy as np
from PIL import Image

from cytopix import png_io
from cytopix import seg_session

data_path = pathlib.Path(__file__).parent / 'data'


def test_dc_to_png_files(tmp_path):
    """Test basic png export"""
    shutil.copy2(data_path / "blood_minimal.rtdc", tmp_path)
    # open session
    ses = seg_session.SegmentationSession(
        path_dc=tmp_path / "blood_minimal.rtdc",
    )
    assert not ses.complete

    # write a label image to the file
    labels = ses.get_labels(240105)
    labels[:] = 0
    labels[:, 0] = 1
    labels[:, 2] = 2
    labels[:, 3] = 3

    ses.write_user_labels(frame=240105, labels=labels)

    index = ses.get_index(240105)

    png_dir = tmp_path / "png"

    # export png files
    png_io.dc_to_png_files(
        dc_path=ses.path_dc,
        png_dir=png_dir,
        export_labels=True,
    )

    # check whether that worked
    assert len(list(png_dir.glob("*.png"))) == 56

    # open a png file and check labels
    im = np.array(Image.open(png_dir / f"image_{index}_label.png"))
    assert np.all(im[:, 0] == int(255 / 3))
    assert np.all(im[:, 1] == 0)
    assert np.all(im[:, 2] == int(255 / 3 * 2))
    assert np.all(im[:, 3] == int(255))


def test_png_files_to_dc(tmp_path):
    """Test converting PNG files to PNG"""
    png_dir = tmp_path / "png"
    dc_path = tmp_path / "data.rtdc"
    png_dir.mkdir()
    # First, create a few grayscale PNG files
    for ii in range(10):
        image = np.zeros((80, 320), dtype=np.uint8)
        image[0, ii] = 10 * ii
        Image.fromarray(image).save(png_dir / f"image_{ii:02d}.png")

    # Then, convert those files to .rtdc
    png_io.png_files_to_dc(
        png_paths=png_dir,
        dc_path=dc_path,
    )

    # open the file
    with dclab.new_dataset(dc_path) as ds:
        assert len(ds) == 10
        assert ds["image"][1][0, 1] == 10
        assert ds["image"][2][0, 2] == 20
        assert ds["image"][3][0, 3] == 30
        assert ds["image"][4][0, 4] == 40
        assert ds["image"][5][0, 5] == 50
        assert ds["image"][6][0, 6] == 60
        assert ds["image"][7][0, 7] == 70
        assert ds["image"][8][0, 8] == 80
        assert ds["image"][9][0, 9] == 90



"""GUI for pixel-based manual segmentation of deformability cytometry images
Copyright (C) 2025 Paul Müller

This program is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation, either version 3 of the License, or (at your
option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program. If not, see <https://www.gnu.org/licenses/>.
"""
from ._version import __version__, __version_tuple__  # noqa: F401



# flake8: noqa
# Form implementation generated from reading ui file './ui/main.ui'
#
# Created by: PyQt6 UI code generator 6.8.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(967, 600)
        self.centralwidget = QtWidgets.QWidget(parent=MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.lineEdit_session = QtWidgets.QLineEdit(parent=self.centralwidget)
        self.lineEdit_session.setReadOnly(True)
        self.lineEdit_session.setObjectName("lineEdit_session")
        self.gridLayout.addWidget(self.lineEdit_session, 1, 1, 1, 1)
        self.lineEdit_source = QtWidgets.QLineEdit(parent=self.centralwidget)
        self.lineEdit_source.setReadOnly(True)
        self.lineEdit_source.setObjectName("lineEdit_source")
        self.gridLayout.addWidget(self.lineEdit_source, 0, 1, 1, 1)
        self.label = QtWidgets.QLabel(parent=self.centralwidget)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.label_2 = QtWidgets.QLabel(parent=self.centralwidget)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 1, 0, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_26 = QtWidgets.QLabel(parent=self.centralwidget)
        self.label_26.setObjectName("label_26")
        self.horizontalLayout.addWidget(self.label_26)
        self.label_frame = QtWidgets.QLabel(parent=self.centralwidget)
        self.label_frame.setObjectName("label_frame")
        self.horizontalLayout.addWidget(self.label_frame)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.widget_labels = QtWidgets.QWidget(parent=self.centralwidget)
        self.widget_labels.setObjectName("widget_labels")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.widget_labels)
        self.horizontalLayout_2.setContentsMargins(-1, 0, -1, 0)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_28 = QtWidgets.QLabel(parent=self.widget_labels)
        self.label_28.setObjectName("label_28")
        self.horizontalLayout_2.addWidget(self.label_28)
        self.label_label_current = QtWidgets.QLabel(parent=self.widget_labels)
        self.label_label_current.setObjectName("label_label_current")
        self.horizontalLayout_2.addWidget(self.label_label_current)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.label_labels_available = QtWidgets.QLabel(parent=self.widget_labels)
        self.label_labels_available.setObjectName("label_labels_available")
        self.horizontalLayout_2.addWidget(self.label_labels_available)
        self.horizontalLayout.addWidget(self.widget_labels)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout.addItem(spacerItem2)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.graphics_widget = GraphicsLayoutWidget(parent=self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.graphics_widget.sizePolicy().hasHeightForWidth())
        self.graphics_widget.setSizePolicy(sizePolicy)
        self.graphics_widget.setObjectName("graphics_widget")
        self.verticalLayout.addWidget(self.graphics_widget)
        self.widget = QtWidgets.QWidget(parent=self.centralwidget)
        self.widget.setObjectName("widget")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.widget)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.label_13 = QtWidgets.QLabel(parent=self.widget)
        self.label_13.setObjectName("label_13")
        self.gridLayout_2.addWidget(self.label_13, 5, 1, 1, 1)
        self.label_20 = QtWidgets.QLabel(parent=self.widget)
        self.label_20.setTextFormat(QtCore.Qt.TextFormat.MarkdownText)
        self.label_20.setObjectName("label_20")
        self.gridLayout_2.addWidget(self.label_20, 9, 0, 1, 1)
        self.label_12 = QtWidgets.QLabel(parent=self.widget)
        self.label_12.setTextFormat(QtCore.Qt.TextFormat.MarkdownText)
        self.label_12.setObjectName("label_12")
        self.gridLayout_2.addWidget(self.label_12, 5, 0, 1, 1)
        self.label_19 = QtWidgets.QLabel(parent=self.widget)
        self.label_19.setObjectName("label_19")
        self.gridLayout_2.addWidget(self.label_19, 8, 1, 1, 1)
        self.label_15 = QtWidgets.QLabel(parent=self.widget)
        self.label_15.setObjectName("label_15")
        self.gridLayout_2.addWidget(self.label_15, 6, 1, 1, 1)
        self.label_22 = QtWidgets.QLabel(parent=self.widget)
        self.label_22.setTextFormat(QtCore.Qt.TextFormat.MarkdownText)
        self.label_22.setObjectName("label_22")
        self.gridLayout_2.addWidget(self.label_22, 10, 0, 1, 1)
        self.label_8 = QtWidgets.QLabel(parent=self.widget)
        self.label_8.setTextFormat(QtCore.Qt.TextFormat.MarkdownText)
        self.label_8.setObjectName("label_8")
        self.gridLayout_2.addWidget(self.label_8, 3, 0, 1, 1)
        self.label_11 = QtWidgets.QLabel(parent=self.widget)
        self.label_11.setObjectName("label_11")
        self.gridLayout_2.addWidget(self.label_11, 4, 1, 1, 1)
        self.label_23 = QtWidgets.QLabel(parent=self.widget)
        self.label_23.setObjectName("label_23")
        self.gridLayout_2.addWidget(self.label_23, 10, 1, 1, 1)
        self.label_24 = QtWidgets.QLabel(parent=self.widget)
        self.label_24.setTextFormat(QtCore.Qt.TextFormat.MarkdownText)
        self.label_24.setObjectName("label_24")
        self.gridLayout_2.addWidget(self.label_24, 11, 0, 1, 1)
        self.label_16 = QtWidgets.QLabel(parent=self.widget)
        self.label_16.setTextFormat(QtCore.Qt.TextFormat.MarkdownText)
        self.label_16.setObjectName("label_16")
        self.gridLayout_2.addWidget(self.label_16, 7, 0, 1, 1)
        self.label_4 = QtWidgets.QLabel(parent=self.widget)
        self.label_4.setTextFormat(QtCore.Qt.TextFormat.MarkdownText)
        self.label_4.setObjectName("label_4")
        self.gridLayout_2.addWidget(self.label_4, 2, 0, 1, 1)
        self.label_25 = QtWidgets.QLabel(parent=self.widget)
        self.label_25.setObjectName("label_25")
        self.gridLayout_2.addWidget(self.label_25, 11, 1, 1, 1)
        self.label_21 = QtWidgets.QLabel(parent=self.widget)
        self.label_21.setObjectName("label_21")
        self.gridLayout_2.addWidget(self.label_21, 9, 1, 1, 1)
        self.label_14 = QtWidgets.QLabel(parent=self.widget)
        self.label_14.setTextFormat(QtCore.Qt.TextFormat.MarkdownText)
        self.label_14.setObjectName("label_14")
        self.gridLayout_2.addWidget(self.label_14, 6, 0, 1, 1)
        self.label_6 = QtWidgets.QLabel(parent=self.widget)
        self.label_6.setTextFormat(QtCore.Qt.TextFormat.MarkdownText)
        self.label_6.setObjectName("label_6")
        self.gridLayout_2.addWidget(self.label_6, 1, 0, 1, 1)
        self.label_5 = QtWidgets.QLabel(parent=self.widget)
        self.label_5.setObjectName("label_5")
        self.gridLayout_2.addWidget(self.label_5, 2, 1, 1, 1)
        self.label_17 = QtWidgets.QLabel(parent=self.widget)
        self.label_17.setObjectName("label_17")
        self.gridLayout_2.addWidget(self.label_17, 7, 1, 1, 1)
        self.label_9 = QtWidgets.QLabel(parent=self.widget)
        self.label_9.setObjectName("label_9")
        self.gridLayout_2.addWidget(self.label_9, 3, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(parent=self.widget)
        self.label_3.setTextFormat(QtCore.Qt.TextFormat.MarkdownText)
        self.label_3.setObjectName("label_3")
        self.gridLayout_2.addWidget(self.label_3, 0, 0, 1, 1)
        self.label_10 = QtWidgets.QLabel(parent=self.widget)
        self.label_10.setTextFormat(QtCore.Qt.TextFormat.MarkdownText)
        self.label_10.setObjectName("label_10")
        self.gridLayout_2.addWidget(self.label_10, 4, 0, 1, 1)
        self.label_7 = QtWidgets.QLabel(parent=self.widget)
        self.label_7.setObjectName("label_7")
        self.gridLayout_2.addWidget(self.label_7, 1, 1, 1, 1)
        self.label_18 = QtWidgets.QLabel(parent=self.widget)
        self.label_18.setTextFormat(QtCore.Qt.TextFormat.MarkdownText)
        self.label_18.setObjectName("label_18")
        self.gridLayout_2.addWidget(self.label_18, 8, 0, 1, 1)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout_2.addItem(spacerItem3, 1, 2, 1, 1)
        self.verticalLayout.addWidget(self.widget)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(parent=MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 967, 23))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(parent=self.menubar)
        self.menuFile.setObjectName("menuFile")
        self.menuHelp = QtWidgets.QMenu(parent=self.menubar)
        self.menuHelp.setObjectName("menuHelp")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(parent=MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.actionOpen_rtdc_file = QtGui.QAction(parent=MainWindow)
        self.actionOpen_rtdc_file.setObjectName("actionOpen_rtdc_file")
        self.actionSegmentRrtdcFile = QtGui.QAction(parent=MainWindow)
        self.actionSegmentRrtdcFile.setObjectName("actionSegmentRrtdcFile")
        self.actionSegmentPngImages = QtGui.QAction(parent=MainWindow)
        self.actionSegmentPngImages.setObjectName("actionSegmentPngImages")
        self.actionQuit = QtGui.QAction(parent=MainWindow)
        self.actionQuit.setObjectName("actionQuit")
        self.actionExportImages = QtGui.QAction(parent=MainWindow)
        self.actionExportImages.setObjectName("actionExportImages")
        self.actionAbout = QtGui.QAction(parent=MainWindow)
        self.actionAbout.setObjectName("actionAbout")
        self.actionSoftware = QtGui.QAction(parent=MainWindow)
        self.actionSoftware.setObjectName("actionSoftware")
        self.actionDocumentation = QtGui.QAction(parent=MainWindow)
        self.actionDocumentation.setObjectName("actionDocumentation")
        self.menuFile.addAction(self.actionSegmentRrtdcFile)
        self.menuFile.addAction(self.actionSegmentPngImages)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionExportImages)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionQuit)
        self.menuHelp.addAction(self.actionAbout)
        self.menuHelp.addAction(self.actionSoftware)
        self.menuHelp.addAction(self.actionDocumentation)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))
        self.label.setText(_translate("MainWindow", "DC file"))
        self.label_2.setText(_translate("MainWindow", "Session"))
        self.label_26.setText(_translate("MainWindow", "Current frame:"))
        self.label_frame.setText(_translate("MainWindow", "--"))
        self.label_28.setText(_translate("MainWindow", "Current label:"))
        self.label_label_current.setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:700\">--</span></p></body></html>"))
        self.label_labels_available.setText(_translate("MainWindow", "<html><head/><body><p>Frame Labels: --</p></body></html>"))
        self.label_13.setText(_translate("MainWindow", "Show previous/next image frame"))
        self.label_20.setText(_translate("MainWindow", "**B**"))
        self.label_12.setText(_translate("MainWindow", "Arrow keys (**← Left** or **→ Right**)"))
        self.label_19.setText(_translate("MainWindow", "Cycle through label visualization modes"))
        self.label_15.setText(_translate("MainWindow", "Zoom in/out"))
        self.label_22.setText(_translate("MainWindow", "**C**"))
        self.label_8.setText(_translate("MainWindow", "**Shift + Alt +Click**"))
        self.label_11.setText(_translate("MainWindow", "Change the current drawing label color"))
        self.label_23.setText(_translate("MainWindow", "Toggle auto contrast"))
        self.label_24.setText(_translate("MainWindow", "\"**.**\" (dot) and \"**-**\" (dash)"))
        self.label_16.setText(_translate("MainWindow", "**Space**"))
        self.label_4.setText(_translate("MainWindow", "**Shift + Click (and drag)**"))
        self.label_25.setText(_translate("MainWindow", "Modify label color saturation"))
        self.label_21.setText(_translate("MainWindow", "Toggle background correction (if available)"))
        self.label_14.setText(_translate("MainWindow", "**Mouse wheel** or **Right-Click (and drag)**"))
        self.label_6.setText(_translate("MainWindow", "**Click (and drag)**"))
        self.label_5.setText(_translate("MainWindow", "Remove pixels from a mask"))
        self.label_17.setText(_translate("MainWindow", "Temporarily hide the mask data"))
        self.label_9.setText(_translate("MainWindow", "Remove an entire mask"))
        self.label_3.setText(_translate("MainWindow", "**Shortcuts**"))
        self.label_10.setText(_translate("MainWindow", "Numbers (**1**, **2**, **3**, **4**, **5**, **6**, or **7**)"))
        self.label_7.setText(_translate("MainWindow", "Create or edit a mask. Holes are filled automatically"))
        self.label_18.setText(_translate("MainWindow", "**V**"))
        self.menuFile.setTitle(_translate("MainWindow", "File"))
        self.menuHelp.setTitle(_translate("MainWindow", "Help"))
        self.actionOpen_rtdc_file.setText(_translate("MainWindow", "Start/continue segmenting an .rtdc file..."))
        self.actionSegmentRrtdcFile.setText(_translate("MainWindow", "Segment .&rtdc file..."))
        self.actionSegmentPngImages.setText(_translate("MainWindow", "Segment .&png images..."))
        self.actionQuit.setText(_translate("MainWindow", "&Quit"))
        self.actionExportImages.setText(_translate("MainWindow", "&Export images..."))
        self.actionAbout.setText(_translate("MainWindow", "About"))
        self.actionSoftware.setText(_translate("MainWindow", "Software"))
        self.actionDocumentation.setText(_translate("MainWindow", "Documentation"))
from pyqtgraph import GraphicsLayoutWidget
